/*
 * Copyright (C) 2001/2002 The Honeynet Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by The Honeynet Project.
 * 4. The name "The Honeynet Project" may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#pragma ident "$Id: sebek.h,v 1.11 2003/11/30 18:37:58 job Exp $"

#ifndef _SEBEK_H
#define _SEBEK_H

#include "config.h"

#ifdef __GNUC__
#define _SYS_VARARGS_H
#define _VARARGS_H
#include <stdarg.h>
#endif

#include <values.h>
#include <inttypes.h>
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/vnode.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/stream.h>
#include <sys/strsubr.h>
#include <sys/stropts.h>
#include <sys/systm.h>
#include <sys/pathname.h>
#include <sys/exec.h>
#include <sys/thread.h>
#include <sys/modctl.h>
#include <sys/syscall.h>
#include <sys/errno.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/autoconf.h>
#include <sys/dirent.h>
#include <sys/kmem.h>
#include <sys/mem.h>
#include <sys/bootconf.h>
#include <sys/reboot.h>
#include <sys/vmparam.h>
#include <sys/var.h>
#include <sys/regset.h>
#include <sys/procfs.h>
#include <sys/tihdr.h>
#include <sys/socket.h>
#include <sys/sockio.h>
#include <sys/fs/ufs_inode.h>
#include <sys/fs/snode.h>
#include <sys/proc/prdata.h>
#include <sys/dlpi.h>
#include <sys/corectl.h>
#include <sys/sad.h>

#if SOL_MINVER >= 6
# include <net/if_types.h>
#endif
#include <net/af.h>
#include <net/route.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip6.h>
#include <sys/byteorder.h>

#include <net/if.h>
#include <inet/common.h>
#include <inet/ip.h>
#include <inet/mib2.h>
#include <inet/tcp.h>

#ifdef __sparcv9
#include <v9/sys/privregs.h>
#endif

#if defined(__sparcv7) || defined(__sparcv8)
#include <v7/sys/privregs.h>
#endif

#include "net.h"

/*
 * Enable the driver debugging code if DEBUG is defined (DEBUG also
 * enables other debugging code, e.g. ASSERT statements).
 */

#ifdef  DEBUG
#define SEBEK_DEBUG
#endif                          /* DEBUG */

/*
 * Debug message control
 * Debug Levels:
 *      0 = no messages
 *      1 = Errors
 *      2 = Subroutine calls & control flow
 *      3 = I/O Data (verbose!)
 * Can be set with adb or in the /etc/system file with
 * "set sebek:sebek_debug=<value>"
 * turn on diagnostics if DEBUG is defined (DEBUG also enables
 * other debugging code, e.g. ASSERT statements).
 */

#define SEBEK_CE_DEBUG1   ((1 << 8) | CE_CONT)
#define SEBEK_CE_DEBUG2   ((2 << 8) | CE_CONT)
#define SEBEK_CE_DEBUG3   ((3 << 8) | CE_CONT)
#define SEBEK_CE_DEBUG4   ((4 << 8) | CE_CONT)
#define SEBEK_LOG         if (sebek_debug) sebek_log

#define FALSE   0
#define TRUE    1

#define OFFADDR(s, n)   (u_char*)((char*)(s) + (int)(n))
#define myABS(x)	((x) < 0 ? -(x) : (x))

#define BUFLEN 1024

#define SBK_VER		1
#define SBK_TYPE_READ	0

#define islower(c)     ((c) >= 'a' && (c) <= 'z')

#define isdigit(c)     ((c) >= '0' && (c) <= '9')

#define isxdigit(c)    (isdigit(c) || ((c) >= 'a' && (c) <= 'f') || \
                        ((c) >= 'A' && (c) <= 'F'))

#define isspace(c)     ((c) == ' ')

#define isascii(c)      ((c & ~0x7F) == 0)

/*
 * stuff found with nm /dev/ksyms
 */

extern int last_module_id;

/*
 * prototypes from sebek.c
 */

extern void sebek_log(int level, const char *fmt, ...);
extern void delayed_init();
extern kmutex_t sebek_lock;

/*
 * prototypes from syscalls.c
 */

extern int64_t (*old_read) (int fd, char *buf, size_t count);
extern int64_t (*old_read32) (int fd, char *buf, size_t count);
extern int64_t new_read(int fd, char *buf, size_t count);
extern int64_t new_read32(int fd, char *buf, size_t count);
extern int (*old_spec_ioctl)(struct vnode *vp, int cmd, intptr_t arg, int mode, struct cred *cr, int *rvalp);
extern int new_spec_ioctl(struct vnode *vp, int cmd, intptr_t arg, int mode, struct cred *cr, int *rvalp);

/*
 * info from str.c
 */

extern struct streamtab sbkinfo;

#ifdef GCC

/*
 * Shared structure for log info
 */

struct sbk_h{
  uint32_t  magic       __attribute__((packed)) ;
  uint16_t  ver         __attribute__((packed)) ;
  uint16_t  type        __attribute__((packed)) ;
  uint32_t  counter     __attribute__((packed)) ;
  uint32_t  time_sec    __attribute__((packed)) ;
  uint32_t  time_usec   __attribute__((packed)) ;
  uint32_t  pid         __attribute__((packed)) ;
  uint32_t  uid         __attribute__((packed)) ;
  uint32_t  fd          __attribute__((packed)) ;
  char      com[12]     __attribute__((packed)) ;
  uint32_t  length      __attribute__((packed)) ;
};

#else

#pragma packed

struct sbk_h{
  uint32_t  magic;
  uint16_t  ver;
  uint16_t  type;
  uint32_t  counter;
  uint32_t  time_sec;
  uint32_t  time_usec;
  uint32_t  pid;
  uint32_t  uid;
  uint32_t  fd;
  char      com[12];
  uint32_t  length;
};
#endif

#endif  /* _SEBEK_H */

