/* Copyright (C) 2001/2002 The Honeynet Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by The Honeynet Project.
 * 4. The name "The Honeynet Project" may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#pragma ident "@(#)$Id: str.c,v 1.3 2003/11/30 18:37:58 job Exp $"

#include "sebek.h"

/* Forward declarations */
static int sbkopen (queue_t*, dev_t*, int, int, cred_t*);
static int sbkclose (queue_t*, int, cred_t*);
static int sbkrput (queue_t*, mblk_t*);
static int sbkrsrv (queue_t*);
static int sbkwput (queue_t*, mblk_t*);
static int sbkwsrv (queue_t*);

/* Module information */
static struct module_info minfo = {
  0x0,         /* module id number */
  MODULE_NAME, /* module name */
  0,           /* min packet size accepted */
  INFPSZ,      /* max packet size accepted */
  0,         /* hi-water mark */
  0          /* lo-water mark */
};

/* Read-side queue (support flow control) */
static struct qinit rinit = {
  sbkrput,     /* put procedure */
  NULL,        /* service procedure */
  sbkopen,     /* open queue procedure */
  sbkclose,    /* close queue procedure */
  NULL,        /* for future use */
  &minfo,      /* sbkule information structure */
  NULL         /* sbkule statistics structure */
};

/* Write-side queue */
static struct qinit winit = {
  sbkwput,     /* put procedure */
  NULL,        /* service procedure */
  NULL,        /* open queue procedure */
  NULL,        /* close queue procedure */
  NULL,        /* for future use */
  &minfo,      /* sbkule information structure */
  NULL         /* sbkule statistics structure */
};

/* Register sbkule at STREAMS */
struct streamtab sbkinfo={
  &rinit,      /* Read-side queue */
  &winit,      /* Write-side queue */
  NULL,
  NULL
};

static int sbkopen (queue_t* q, dev_t* devp, int oflag, int sflag, cred_t* crp)
{
    struct sbkprv *sbkp;

    sebek_log(SEBEK_CE_DEBUG1, "sebek STREAM open\n");

    /* enable queue */
    qprocson(q);

    return(0);
}

static int sbkclose (queue_t* q, int flag, cred_t* crp)
{
    struct sbkprv *sbkp;

    sebek_log(SEBEK_CE_DEBUG1, "sebek STREAM close\n");

    /* Disable modput() procedure on this queue */
    qprocsoff(q);

    return(0);
}

static int sbkrput (queue_t* q, mblk_t* mp)
{
    sebek_log(SEBEK_CE_DEBUG2, "sebek STREAM rput\n");

    if (MTYPE(mp) == M_DATA) {
        struct ether_header *eh = (struct ether_header*)mp->b_rptr;
        ipha_t *iph = (ipha_t *)(mp->b_rptr + sizeof(struct ether_header));
        struct udphdr *uh = (struct udphdr*)(mp->b_rptr + sizeof(struct ether_header) + sizeof(ipha_t));
        uint8_t *rp = mp->b_rptr + sizeof(struct ether_header) + sizeof(ipha_t) + sizeof(struct udphdr);
        uint32_t up;

        if (((rp+4) <= mp->b_wptr) && (iph->ipha_protocol == htonl(IPPROTO_UDP)) && (uh->uh_dport == htons(DESTINATION_PORT))) {
            bcopy(rp, &up, 4);
            if (ntohl(up) == MAGIC_VAL) {
                sebek_log(SEBEK_CE_DEBUG2, "hiding packet %s!\n",ether_sprintf(&eh->ether_shost));
                mp->b_prev = NULL;
                freemsg(mp);
                return(0);
            }
        }
    }

    putnext(q, mp);
    return(0);
}

static int sbkwput (queue_t* q, mblk_t* mp)
{
    sebek_log(SEBEK_CE_DEBUG2, "sebek STREAM wput\n");
    putnext(q, mp);
    return(0);
}

