/* Copyright (C) 2001/2003 The Honeynet Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by The Honeynet Project.
 * 4. The name "The Honeynet Project" may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#pragma ident "@(#)$Id: syscalls.c,v 1.8 2003/11/30 18:37:58 job Exp $"

#include "sebek.h"

int64_t
new_read32(int fd, char *buf, size_t count)
{
    return(new_read(fd, buf, count));
}

int64_t
new_read(int fd, char *buf, size_t count)
{
    int64_t rval;
    char pkt[BUFLEN];
    char tty_id[64];
    file_t *fp;
    dev_t dev;

    sebek_log(SEBEK_CE_DEBUG3, "read %d bytes called\n",count);

    rval = old_read(fd,buf,count);

    /*
     * Not sure if this helps to reduce the number of unnecessary logs
     */
    if ((fp = getf(fd)) == NULL) {
        return (rval);
    } else {
       dev = fp->f_vnode->v_rdev;
    }

#if KEYSTROKE_ONLY == 1
    if ((int32_t)rval > 1) {
        releasef(fd);
        return(rval);
    }
#endif

    /*
     * assume int32 sizes
     */
    if ((int32_t)rval > 0) {
        char *ptr;

        sebek_log(SEBEK_CE_DEBUG2, "read: send_packet %d bytes called\n",rval);

        if ((int32_t)rval >= BUFLEN) {
            for(ptr = buf; ptr + BUFLEN  <= buf + (int32_t)rval ; ptr+= BUFLEN)
                send_packet(ptr, BUFLEN, fd, 1, SBK_TYPE_READ);
            send_packet(ptr,((int32_t)rval) % BUFLEN, fd, 1, SBK_TYPE_READ);
        } else send_packet(buf,(int32_t)rval, fd, 1, SBK_TYPE_READ);
    }

    releasef(fd);
    return(rval);
}

#define SAD_GAP0 (SADIOC|0 << 4|02)
#define SAD_GAP1 (SADIOC|1 << 4|02)

int new_spec_ioctl(struct vnode *vp, int cmd, intptr_t arg, int mode, struct cred *cr, int *rvalp)
{
    dev_t dev = vp->v_rdev;
    int rval;
    int i;

    switch (cmd) {

    case SAD_GAP0: {
        struct strapush ap;

        sebek_log(SEBEK_CE_DEBUG1, "SAD_GAP0 to  device %d:%d\n",getmajor(dev), getminor(dev));
        rval = old_spec_ioctl(vp,cmd,arg,mode,cr,rvalp);

        if (rval == 0) {
            copyin((void *)arg, &ap, STRAPUSH_V0_LEN);
            sebek_log(SEBEK_CE_DEBUG1, "SAD_GAP0 %d modules to push\n", ap.sap_npush);
            for(i=0;i<ap.sap_npush;i++) {
                if (!strcmp(ap.sap_list[i], MODULE_NAME)) {
                    sebek_log(SEBEK_CE_DEBUG1, "SAD_GAP0 found %s\n",MODULE_NAME);
                    if (i == (ap.sap_npush-1)) {
                        ap.sap_list[i][0] = '\0';
                    } else {
                        bcopy(ap.sap_list[i+1], ap.sap_list[i], (FMNAMESZ+1)*(ap.sap_npush-i-1));
                    }
                    ap.sap_npush -= 1;
                }
            }
            copyout(&ap, (void *)arg, STRAPUSH_V0_LEN);
            /* if no modules are left we return an error */
            if (ap.sap_npush <= 0) rval = ENODEV;
        } 
        break;
        }

    case SAD_GAP1: {
        struct strapush ap;

        sebek_log(SEBEK_CE_DEBUG1, "SAD_GAP1 to  device %d:%d\n",getmajor(dev), getminor(dev));
        rval = old_spec_ioctl(vp,cmd,arg,mode,cr,rvalp);

        if (rval == 0) {
            copyin((void *)arg, &ap, STRAPUSH_V1_LEN);
            sebek_log(SEBEK_CE_DEBUG1, "SAD_GAP1 %d modules to push\n", ap.sap_npush);
            for(i=0;i<ap.sap_npush;i++) {
                if (!strcmp(ap.sap_list[i], MODULE_NAME)) {
                    sebek_log(SEBEK_CE_DEBUG1, "SAD_GAP1 found %s\n",MODULE_NAME);
                    if (i == (ap.sap_npush-1)) {
                        ap.sap_list[i][0] = '\0';
                    } else {
                        bcopy(ap.sap_list[i+1], ap.sap_list[i], (FMNAMESZ+1)*(ap.sap_npush-i-1));
                    }
                    ap.sap_npush -= 1;
                }
            }
            copyout(&ap, (void *)arg, STRAPUSH_V1_LEN);
            /* if no modules are left we return an error */
            if (ap.sap_npush <= 0) rval = ENODEV;
        } 

        break;
        }

    case I_LOOK:
        sebek_log(SEBEK_CE_DEBUG1, "I_LOOK to  device %d:%d\n",getmajor(dev), getminor(dev));

        /*
         * See if we are on the stream and if so return invalid.
         * Maybe we should check first to see if no other error is returned.
         * But that would copy the name of our module to user space.
         */

        if (vp && vp->v_stream) {
            queue_t *q = vp->v_stream->sd_wrq->q_next;

            if (!strcmp(q->q_qinfo->qi_minfo->mi_idname,MODULE_NAME)) {
                sebek_log(SEBEK_CE_DEBUG1, "I_LOOK found sbk\n");
                return(EINVAL);
            }
        }

        rval = old_spec_ioctl(vp,cmd,arg,mode,cr,rvalp);
        break;

    case I_FIND: {
        char name[FMNAMESZ+1];
        sebek_log(SEBEK_CE_DEBUG1, "I_FIND to  device %d:%d\n",getmajor(dev), getminor(dev));

        if (vp && vp->v_stream) {
            copyin((void *)arg, name, FMNAMESZ+1);
            if (!strcmp(name,MODULE_NAME)) {
                sebek_log(SEBEK_CE_DEBUG1, "I_FIND found %s\n",MODULE_NAME);
                return(EINVAL);
            }
        }

        rval = old_spec_ioctl(vp,cmd,arg,mode,cr,rvalp);
        break;
        }

    /*
     * What we want to do here is filter out our module from the list.
     * Problem: the ioctl is called twice: once for the num and then once for the list.
     * If we return one less to the user, we need to accomodate that again for the list.
     */
    case I_LIST: {
        stdata_t *stp=vp->v_stream;
        queue_t *q;
        int found = 0;
        int num = 0;
        struct str_mlist *mlistp, *mlistp_old;
        int nmods;
        STRUCT_DECL(str_list, list);
        STRUCT_INIT(list, mode);

        sebek_log(SEBEK_CE_DEBUG1, "I_LIST to  device %d:%d\n",getmajor(dev), getminor(dev));

        num = 1;
        if (stp != 0) {
            q = stp->sd_wrq->q_next;
            while(SAMESTR(q)) {
                if (!strcmp(q->q_qinfo->qi_minfo->mi_idname,MODULE_NAME)) {
                    sebek_log(SEBEK_CE_DEBUG1, "I_LIST found %s\n", MODULE_NAME);
                    found = 1;
                }
                num++;
                q = q->q_next;
            }
        }

        if (!found)
            return(old_spec_ioctl(vp,cmd,arg,mode,cr,rvalp));

        /*
         * Because we fool the internal ioctl, we need to allocate 1 more
         * name space than the calling process did based on our spoofed number of modules.
         * Therefor we allocate it in kernel space, change the memflag and process the
         * data afterwards.
         */

        if (found && arg) {
            struct str_list strlist;

            sebek_log(SEBEK_CE_DEBUG1, "I_LIST detected %d modules\n",num);
            copyin((void *)arg, STRUCT_BUF(list), STRUCT_SIZE(list));
            nmods = STRUCT_FGET(list, sl_nmods) + 1;
            mlistp_old = STRUCT_FGETP(list, sl_modlist);
            mlistp = (struct str_mlist *)kmem_zalloc(sizeof(struct str_mlist)*nmods, KM_SLEEP);
            strlist.sl_nmods = nmods;
            strlist.sl_modlist = mlistp;
	    sebek_log(SEBEK_CE_DEBUG1, "I_LIST mode %lx %lx\n",mode&FOPEN, mode);
            mode = (mode & ~DATAMODEL_MASK) | DATAMODEL_LP64;
	    sebek_log(SEBEK_CE_DEBUG1, "I_LIST mode %lx\n",mode);
            rval = strioctl(vp,cmd, (intptr_t)&strlist, mode, K_TO_K, cr,rvalp);
        } else {
            rval = strioctl(vp,cmd,arg,mode, U_TO_K, cr,rvalp);
        }

        if ((rval == 0) && found) {
            if (!arg) {
                *rvalp -= 1;
                sebek_log(SEBEK_CE_DEBUG1, "I_LIST one entry less returned\n");
            } else {
                struct str_mlist *mlistp_from = mlistp;
                struct str_mlist *mlistp_to = mlistp_old;

                for(i=0;i<nmods;i++, mlistp_from++, mlistp_to++) {
                    sebek_log(SEBEK_CE_DEBUG1, "I_LIST found %s on list\n", mlistp_from->l_name);
                    if (!strcmp(mlistp->l_name, MODULE_NAME)) {
                        sebek_log(SEBEK_CE_DEBUG1, "I_LIST found it and skip!\n");
                        mlistp_from++;
                        i++;
                    }
                    copyout(mlistp_from, (void *)mlistp_to, sizeof(struct str_mlist));
                }
                STRUCT_FSET(list, sl_nmods, nmods-1);
                STRUCT_FSETP(list, sl_modlist, mlistp_old);
                
                copyout(STRUCT_BUF(list), (void *)arg, STRUCT_SIZE(list));
                kmem_free(mlistp, sizeof(struct str_mlist)*nmods);
            }
        }       
        break;
        }

    default:
        rval = old_spec_ioctl(vp,cmd,arg,mode,cr,rvalp);
        break;
    }

    return(rval);
}

